//--------------------------------------------------------------------------------------------------
// File : VertexTextureFetchingFog.fx
//
// Date : Nov. 03, 2008
// Version : 1.0
// Author : Pocol
//--------------------------------------------------------------------------------------------------

//
// Global Variables
//
float4x4 World;
float4x4 View;
float4x4 Projection;

// TODO: add effect parameters here.
float3 CameraPosition;
float3 LightPosition;
float3 DiffuseColor;
float3 SpecularColor;
float SpecularPower;

float2 FogCoord;
float4 FogColor;
float2 FogTranslate;

texture DecalMap;
texture FogMap;
sampler DecalSmp = sampler_state
{
	Texture = <DecalMap>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
};
sampler FogSmp = sampler_state
{
	Texture = <FogMap>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
};


//
// VertexShaderInput
//
struct VertexShaderInput
{
    float4 Position : POSITION0;

    // TODO: add input channels such as texture
    // coordinates and vertex colors here.
	float3 Normal : NORMAL;
	float2 TexCoord : TEXCOORD0;
};

//
// VertexShaderOutput
//
struct VertexShaderOutput
{
    float4 Position : POSITION0;

    // TODO: add vertex shader outputs such as colors and texture
    // coordinates here. These values will automatically be interpolated
    // over the triangle, and provided as input to your pixel shader.
	float3 Normal : NORMAL;
	float2 TexCoord : TEXCOORD0;
	float4 ObjPosition : TEXCOORD1;
	float Fog : TEXCOORD2;
	float Density : TEXCOORD3;
};

//--------------------------------------------------------------------------------------------------
// Name : VertexShaderFunction()
// Desc : _VF[_
//--------------------------------------------------------------------------------------------------
VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output = (VertexShaderOutput)0;

	//@Wϊ
    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

    // TODO: add your vertex shader code here.
	output.ObjPosition = input.Position;
	output.TexCoord = input.TexCoord;
	output.Normal = input.Normal;

	//@eNX`W
	float2 uv = input.TexCoord + FogTranslate;

	//@eNX`Zxo
	float density = tex2Dlod( FogSmp, float4( uv, 0, 0 ) ).x;
	output.Density = density * 0.75f + 0.25f;
	
	//@tHOWZo
	output.Fog = FogCoord.x + input.Position.y * FogCoord.y;
	
    return output;
}

//-------------------------------------------------------------------------------------------------
// Name : HalfLambert()
// Desc : n[to[gCeBO
//-------------------------------------------------------------------------------------------------
float4 HalfLambert(float3 N, float3 L, float3 H)
{
	float3 Ambient = DiffuseColor * 0.3;
	float3 Diffuse = DiffuseColor * ( max( dot( N, L ), 0 ) * 0.5 + 0.5 );
	float3 Specular = SpecularColor * pow( max( dot( N, H ), 0 ), SpecularPower );
	return float4( Ambient + Diffuse + Specular, 1.0f );
}

//--------------------------------------------------------------------------------------------------
// Name : PixelShaderFunction()
// Desc : sNZVF[_
//--------------------------------------------------------------------------------------------------
float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
    // TODO: add your pixel shader code here.
	float3 N = normalize( input.Normal );
	float3 L = normalize( LightPosition - input.ObjPosition.xyz );
	float3 E = normalize( CameraPosition - input.ObjPosition.xyz );
	float3 H = normalize( L + E );

	//@CeBO
	float4 output = HalfLambert( N, L, H );

	//@eNX`
	output *= tex2D( DecalSmp, input.TexCoord );

	//@ZxlFZo
	float4 fog_color = FogColor * input.Density;

	//@tHOFƃIuWFNgF`
	output = lerp( fog_color, output, input.Fog );

    return output;
}

//
// Technique
//
technique Technique1
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }
}
